(define-module (alterator gettext)
  :use-module (srfi srfi-1)
  :use-module (alterator algo)
  :use-module (alterator str)
  :export (make-translator
           make-dictionary
           dictionary-for-each
           make-translator2))

(dynamic-call "scm_init_gettext"
	      (dynamic-link "libguile-gettext.so"))

;;create path like /usr/share/locale/ru/LC_MESSAGE/package.mo
(define (make-system-path item package)
  (string-append "/usr/share/locale/" item "/LC_MESSAGES/" package ".mo"))

;;create path like po/ru.mo
(define (make-local-path item)
  (and (fluid-ref use-local-files)
       (string-append "po/" item ".mo")))

(define (search-mo package candidates)
  (find read-access?
        (fold (lambda (x y);;made full-path, both for local and global paths
                (append y
                        (list (make-local-path x)
                              (make-system-path x package))))
              '()
              (fold (lambda (x y) ;;append short names to expand searching
                          (append y (list x (safe-substring x 0 2))))
                    '()
                    candidates))))

(define (load-dictionary mo-path)
  (let ((dict (make-hash-table 10)))
    (for-each (lambda (item)
                (hash-set! dict (car item) (cdr item)))
              (or (load-mo mo-path) '()))
    dict))

(define (search-dictionary word dict)
  (if dict (hash-ref dict word word) word))

(define (make-dictionary package candidates)
  (let ((full-path (search-mo package candidates)))
       (and full-path (load-dictionary full-path))))

(define (make-translator2 dict)
  (lambda(str) (search-dictionary str dict)))

(define (make-translator package candidates)
  (make-translator2 (make-dictionary package candidates)))

(define (dictionary-for-each proc d)
  (hash-fold (lambda(x y res) (proc x y) #f) #f d))
